package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.TenantInfoDao;
import com.ym.mec.biz.dal.dto.TenantConfigDto;
import com.ym.mec.biz.dal.dto.TenantInfoDto;
import com.ym.mec.biz.dal.dto.TenantProductInfoDto;
import com.ym.mec.biz.dal.entity.TenantConfig;
import com.ym.mec.biz.dal.entity.TenantInfo;
import com.ym.mec.biz.dal.entity.TenantProductInfo;
import com.ym.mec.biz.dal.vo.TenantInfoInfoPageVo;
import com.ym.mec.biz.service.TenantConfigService;
import com.ym.mec.biz.service.TenantInfoService;
import com.ym.mec.biz.service.TenantProductInfoService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Consumer;

@Service
public class TenantInfoServiceImpl extends ServiceImpl<TenantInfoDao, TenantInfo> implements TenantInfoService {

    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private TenantProductInfoService tenantProductInfoService;
    @Autowired
    private TenantConfigService tenantConfigService;

    /**
     * 新增机构
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addTenantInfo(TenantInfoDto dto) {
        //校验手机号是否唯一
        checkPhone(dto);

        //添加机构基本信息
        Integer userId = Optional.ofNullable(sysUserFeignService.queryUserInfo())
                .map(SysUser::getId)
                .orElseThrow(() -> new BizException("用户信息获取失败"));
        TenantInfo tenantInfo = new TenantInfo();
        BeanUtils.copyProperties(dto, tenantInfo);
        tenantInfo.setPayState(0);
        tenantInfo.setState(2);
        tenantInfo.setCreatedBy(userId);
        tenantInfo.setCreatedTime(new Date());
        baseMapper.insert(tenantInfo);

        //机构id
        Integer tenantId = tenantInfo.getId();
        //添加机构产品信息
        setIdByApply(tenantId, dto.getProductInfo(), dto.getProductInfo()::setTenantId,
                tenantProductInfoService::addTenantProduct);
        //添加机构配置
        setIdByApply(tenantId, dto.getConfig(), dto.getConfig()::setTenantId, tenantConfigService::addConfig);
    }

    /**
     * 修改机构信息
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateTenantInfo(TenantInfoDto dto) {
        //机构id
        Integer tenantId = Optional.ofNullable(dto)
                .map(TenantInfoDto::getId)
                .orElseThrow(() -> new BizException("机构信息不能为空!"));

        //查询机构信息
        TenantInfo tenantInfo = this.getById(tenantId);
        if (Objects.isNull(tenantInfo)) {
            throw new BizException("未找到该机构信息!");
        }

        //机构状态 1已缴费，并且 机构注册的手机号与本次修改后的手机号不同，就证明本次修改了手机号 则需要修改机构的账号信息
        if (1 == tenantInfo.getPayState() && !Objects.equals(tenantInfo.getPhone(), dto.getPhone())) {
            //校验修改后的手机号是否是唯一
            checkPhone(dto);
            //修改机构用户手机号(登录账号)信息
            SysUser tenantUser = sysUserFeignService.queryUserByMobile(tenantInfo.getPhone());
            tenantUser.setPhone(dto.getPhone());
            sysUserFeignService.updateSysUser(tenantUser);
        }

        //机构状态 0未缴费
        if (0 == tenantInfo.getPayState()) {
            //修改产品
            setIdByApply(tenantId, dto.getProductInfo(), dto.getProductInfo()::setTenantId,
                    tenantProductInfoService::updateTenantProduct);
        }

        //修改机构基础信息
        Integer userId = Optional.ofNullable(sysUserFeignService.queryUserInfo())
                .map(SysUser::getId)
                .orElseThrow(() -> new BizException("用户信息获取失败"));
        Date now = new Date();
        BeanUtils.copyProperties(dto, tenantInfo);
        tenantInfo.setUpdatedBy(userId);
        tenantInfo.setUpdatedTime(now);
        this.updateById(tenantInfo);

        //修改机构配置
        setIdByApply(tenantId, dto.getConfig(), dto.getConfig()::setTenantId, tenantConfigService::updateConfig);
    }

    /**
     * 查询单个机构详情
     *
     * @param tenantId 机构id
     */
    @Override
    public TenantInfoDto queryTenantInfo(Integer tenantId) {
        //查询机构基础信息
        TenantInfo tenantInfo = Optional.ofNullable(tenantId)
                .map(baseMapper::selectById)
                .orElseThrow(() -> new BizException("未查询到机构信息[" + "tenantId" + "]"));
        TenantInfoDto tenantInfoDto = new TenantInfoDto();
        BeanUtils.copyProperties(tenantInfo, tenantInfoDto);
        //机构产品信息
        TenantProductInfo productInfo = tenantProductInfoService.getOne(new QueryWrapper<TenantProductInfo>()
                .eq("tenant_id_", tenantId));
        setTenantInfo(productInfo, new TenantProductInfoDto(), tenantInfoDto::setProductInfo);
        //机构配置信息
        TenantConfig tenantConfig = tenantConfigService.getOne(new QueryWrapper<TenantConfig>()
                .eq("tenant_id_", tenantId));
        setTenantInfo(tenantConfig, new TenantConfigDto(), tenantInfoDto::setConfig);
        return tenantInfoDto;
    }

    /**
     * 启用/停用
     *
     * @param id    机构id
     * @param state 机构状态 0草稿 1启动 2停用
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void opsTenantState(Integer id, Integer state) {
        if (Objects.nonNull(state)) {
            if (state == 1 || state == 2) {
                baseMapper.opsTenantState(id, state);
                return;
            }
        }
        throw new BizException("机构状态参数异常!");
    }

    /**
     * 分页查询
     *
     * @param param 查询条件
     * @return
     */
    @Override
    public PageInfo<TenantInfoInfoPageVo> queryPage(Map<String, Object> param) {
        Page<TenantInfoInfoPageVo> pageInfo = PageUtil.getPageInfo(param);
        pageInfo.setAsc("a.id_");
        return PageUtil.pageInfo(baseMapper.queryPage(pageInfo, param));
    }

    /**
     * 校验手机号
     */
    private void checkPhone(TenantInfoDto dto) {
        SysUser sysUser = Optional.ofNullable(dto)
                .map(TenantInfoDto::getPhone)
                .map(sysUserFeignService::queryUserByMobile)
                .orElse(null);
        if (Objects.nonNull(sysUser)) {
            throw new BizException("该手机号已被注册!");
        }
    }

    /**
     * set机构Id 并执行一个操作
     *
     * @param tenantId  机构id
     * @param clazz     setId的类
     * @param setOption set构造函数
     * @param action    需要执行的操作
     */
    private <T> void setIdByApply(Integer tenantId, T clazz, Consumer<Integer> setOption, Consumer<T> action) {
        Optional.ofNullable(tenantId)
                .ifPresent(setOption);
        action.accept(clazz);
    }

    /**
     * 给机构各关联配置赋值
     *
     * @param source 原对象
     * @param target 转换的对象
     * @param action 执行动作
     */
    private <T> void setTenantInfo(Object source, T target, Consumer<T> action) {
        if (Objects.nonNull(source)) {
            BeanUtils.copyProperties(source, target);
            action.accept(target);
        }
    }

}